import _ from 'underscore'
import queryString from 'query-string'
import cookie from '../managers/cookie'

// import request from 'request-promise'
import axios from 'axios'

class BaseApi {

    constructor() {
        this._METHOD_GET = 'get'
        this._METHOD_POST = 'post'
        this._METHOD_PATCH = 'patch'
        this._METHOD_DELETE = 'delete'

        let extension = process.env.NODE_ENV === 'production' ? 'com/' : 'xyz/'
        this._BASE_URL = 'https://auth.marketplays.' + extension
    }

    /**
     * Get api path
     * @return {string}
     */
    getApiPath() {
    }

    /** Assign an object */
    getDataObject() {
    }

    /**
     * create response
     * @param {boolean} isSuccess
     * @param {function} response
     * @return {{success: *, data: *}}
     */
    static createResponse(isSuccess, response) {
        return {success: isSuccess, data: response}
    }

    /**
     * Get record list
     * @param {object} params
     * @return {Promise}
     */
    getRecordsList(params) {
        let additionalParams = params ? '?' + queryString.stringify(params) : ''
        let _this = this
        return new Promise(function (resolve, reject) {
            _this.makeApiRequest(_this._METHOD_GET, _this.getApiPath() + additionalParams)
                .then(result => {
                    let newArray = []
                    if (result.total !== 0) {
                        let iThis = _this
                        _.map(result.data.records, function (record) {
                            newArray.push(iThis.getDataObject().parseDataFromResourceServer(record))
                        })
                        result.data.records = newArray
                        resolve(result.data)
                    } else {
                        resolve(result)
                    }
                })
                .catch(error => {
                    reject(error)
                })
        })
    }

    /**
     * Get single record
     * @param {number} recordId
     * @return {Promise}
     */
    getSingleRecord(recordId) {
        let _this = this
        return new Promise(function (resolve, reject) {
            _this.makeApiRequest(_this._METHOD_GET, _this.buildUrlForSingleRecord(recordId), [])
                .then(result => {
                    resolve(_this.getDataObject().parseDataFromResourceServer(result.data))
                })
                .catch(error => {
                    reject(error)
                })
        })
    }

    /**
     * Create record
     * @param {object} data
     * @return {Promise}
     */
    createRecord(data) {
        let _this = this
        return new Promise(function (resolve, reject) {
            _this.makeApiRequest(_this._METHOD_POST, _this.getApiPath(), data)
                .then(result => {
                    resolve(_this.getDataObject().parseDataFromResourceServer(result.data))
                })
                .catch(error => {
                    reject(error)
                })
        })
    }

    /**
     * Update record
     * @param {number} recordId
     * @param {object} data
     * @return {Promise}
     */
    updateRecord(recordId, data) {
        let _this = this
        return new Promise(function (resolve, reject) {
            _this.makeApiRequest(_this._METHOD_PATCH, _this.buildUrlForSingleRecord(recordId), data)
                .then(result => {
                    resolve(_this.getDataObject().parseDataFromResourceServer(result.data))
                })
                .catch(error => {
                    reject(error)
                })
        })
    }

    /**
     * Delete record
     * @param {number} recordId
     * @return {Promise}
     */
    deleteRecord(recordId) {
        let _this = this
        return new Promise(function (resolve, reject) {
            _this.makeApiRequest(_this._METHOD_DELETE, _this.buildUrlForSingleRecord(recordId), [])
                .then(result => {
                    resolve(result)
                })
                .catch(error => {
                    reject(error)
                })
        })
    }

    /**
     * Build url for single record
     * @param {number} recordId
     * @return {string}
     */
    buildUrlForSingleRecord(recordId) {
        return this.getApiPath() + '/' + recordId
    }

    /**
     * Make api request
     * @param {string} method
     * @param {string} url
     * @param {array} data
     * @return {Promise}
     */
    makeApiRequest(method, url, data) {
        return this.callApi(method, 'api/' + url, data)
    }

    /**
     * Call api
     * @param {string} method
     * @param {string} url
     * @param {array} data
     * @return {Promise}
     */
    callApi(method, url, data) {
        const config = {
            method: method,
            url: this._BASE_URL + url,
            data: data,
            headers: {
                'Accept': 'application/json',
                'HTTP_X_MP_LANGUAGE': 'en',
                'Content-Type': 'application/x-www-form-urlencoded'
            }
        }
        // config.headers['Authorization'] = 'Bearer YzY1OTU5M2YzYTQ4OGNhYjE3NDc2MWQ3YjQyNDdlNjAxYTI2MzJiOGU5ZDNkNGZlNzAyZDZhNWQ4YzI4ZmNhYQ'
        if (cookie.getCurrentToken()) {
            config.headers['Authorization'] = 'Bearer ' + cookie.getCurrentToken()
        }
        return new Promise(function (resolve, reject) {
            axios.request(config)
                .then(function (response) {
                    resolve(response)
                })
                .catch(err => {
                    reject(err.response)
                })
        })
    }
}

export default BaseApi
